<?php

const SIMPLERWC_PAYLOAD_PREFIX = "simplerwc-payload";

const SIMPLERWC_BTNPOS_PRODUCT_PAGE = "simpler-product-page";
const SIMPLERWC_BTNPOS_PRODUCT_SHORTCODE = "simpler-product-custom";
const SIMPLERWC_BTNPOS_CART_SHORTCODE = "simpler-cart-custom";
const SIMPLERWC_BTNPOS_BEFORE_CHECKOUT = "simpler-before-checkout-form";
const SIMPLERWC_BTNPOS_PROCEED_TO_CHECKOUT = "simpler-proceed-to-checkout";
const SIMPLERWC_BTNPOS_MINICART = "simpler-mini-cart";

const SIMPLERWC_SEPARATOR_NONE = "simplerwc-separator-none";
const SIMPLERWC_SEPARATOR_TOP = "simplerwc-separator-top";
const SIMPLERWC_SEPARATOR_BOTTOM = "simplerwc-separator-bottom";

add_action('init', 'simplerwc_load_visual_hook_filters');
add_action('init', 'simplerwc_load_custom_shortcodes');

function simplerwc_load_custom_shortcodes()
{
    add_shortcode('simpler-product-checkout', 'simplerwc_product_checkout_shortcode');
    add_shortcode('simpler-cart-checkout', 'simplerwc_cart_checkout_shortcode');
}

function simplerwc_load_visual_hook_filters()
{
    add_filter(
        get_option('simplerwc_product_button_placement', 'woocommerce_after_add_to_cart_quantity'),
        'simplerwc_add_checkout_after_main_content',
        10
    );

    add_filter(
        'woocommerce_widget_shopping_cart_before_buttons',
        'simplerwc_render_mini_cart',
        10
    );

    add_filter(
        get_option('simplerwc_cart_page_button_placement'),
        'simplerwc_render_proceed_to_checkout',
        10
    );

    add_filter(
        get_option('simplerwc_checkout_page_button_placement'),
        'simplerwc_render_before_checkout_form',
        10
    );
}

function simplerwc_product_checkout_shortcode($atts)
{
    if (!simplerwc_should_render_button()) {
        return;
    }

    $a = shortcode_atts(['product_id' => NULL], $atts);
    if (!isset($atts['product_id'])) {
        global $product;
        if (!$product) {
            return '';
        }
    } else {
        $product = \wc_get_product($a['product_id']);
    }

    if (!($product instanceof WC_Product)) {
        return '';
    }

    $target = uniqid(SIMPLERWC_PAYLOAD_PREFIX);
    $content = simplerwc_simple_button($target, "Quick Buy", SIMPLERWC_BTNPOS_PRODUCT_SHORTCODE);
    $content .= simplerwc_cart_contents_tag($target, simplerwc_prepare_product($product));
    return $content;
}

function simplerwc_cart_checkout_shortcode($atts)
{
    if (!simplerwc_should_render_button()) {
        return '';
    }

    $cart = WC()->cart;
    if (is_null($cart) || !($cart instanceof WC_Cart)) {
        return '';
    }

    $target = uniqid(SIMPLERWC_PAYLOAD_PREFIX);
    $content = simplerwc_simple_button($target, "Quick Checkout", SIMPLERWC_BTNPOS_CART_SHORTCODE);
    $content .= simplerwc_cart_contents_tag($target, simplerwc_prepare_cart($cart));
    return $content;
}

function simplerwc_add_checkout_after_main_content()
{
    if (!simplerwc_should_render_button()) {
        return;
    }

    if (!get_option('simpler_auto_render_product_button')) {
        return;
    }

    $separatorPos = [
        'woocommerce_before_add_to_cart_form' => SIMPLERWC_SEPARATOR_BOTTOM,
        'woocommerce_before_add_to_cart_quantity' => SIMPLERWC_SEPARATOR_BOTTOM,
        'woocommerce_after_add_to_cart_quantity' => SIMPLERWC_SEPARATOR_BOTTOM,
        'woocommerce_after_add_to_cart_form' => SIMPLERWC_SEPARATOR_TOP,
        'woocommerce_after_add_to_cart_button' => SIMPLERWC_SEPARATOR_TOP,
        'woocommerce_after_single_product_summary' => SIMPLERWC_SEPARATOR_NONE
    ][get_option('simplerwc_product_button_placement')];

    global $product;
    $target = uniqid(SIMPLERWC_PAYLOAD_PREFIX);
    echo simplerwc_simple_button($target, "Quick Buy", SIMPLERWC_BTNPOS_PRODUCT_PAGE, $separatorPos);
    echo simplerwc_cart_contents_tag($target, simplerwc_prepare_product($product));
}

function simplerwc_render_mini_cart()
{
    if (!simplerwc_should_render_button()) {
        return;
    }

    if (!get_option('simplerwc_auto_render_minicart_button')) {
        return;
    }

    $cart          = WC()->cart;
    if (is_null($cart) || !($cart instanceof WC_Cart)) {
        return;
    }

    $target = uniqid(SIMPLERWC_PAYLOAD_PREFIX);
    echo simplerwc_simple_button($target, "Quick Checkout", SIMPLERWC_BTNPOS_MINICART);
    echo simplerwc_cart_contents_tag($target, simplerwc_prepare_cart($cart));
}

function simplerwc_render_proceed_to_checkout()
{
    if (!simplerwc_should_render_button()) {
        return;
    }

    if (!get_option('simpler_auto_render_cart_button')) {
        return;
    }

    $cart          = WC()->cart;
    if (is_null($cart) || !($cart instanceof WC_Cart)) {
        return;
    }

    $target        = uniqid(SIMPLERWC_PAYLOAD_PREFIX);
    echo simplerwc_simple_button($target, "Quick Checkout", SIMPLERWC_BTNPOS_PROCEED_TO_CHECKOUT, SIMPLERWC_SEPARATOR_BOTTOM);
    echo simplerwc_cart_contents_tag($target, simplerwc_prepare_cart($cart));
}

function simplerwc_render_before_checkout_form()
{
    if (!simplerwc_should_render_button()) {
        return;
    }

    if (!get_option('simpler_auto_render_cart_button')) {
        return;
    }

    $cart          = WC()->cart;
    if (is_null($cart) || !($cart instanceof WC_Cart)) {
        return;
    }

    $target        = uniqid(SIMPLERWC_PAYLOAD_PREFIX);
    echo simplerwc_marquee_button($target, 'Quick Checkout', SIMPLERWC_BTNPOS_BEFORE_CHECKOUT);
    echo simplerwc_cart_contents_tag($target, simplerwc_prepare_cart($cart));
}

function simplerwc_simple_button($id, $text, $positionId, $separatorPos = SIMPLERWC_SEPARATOR_NONE)
{
    $html = simplerwc_button_style($positionId);
    $html .= '<div class="simpler-container ' . $positionId . '">';
    $html .= '<div class="simpler-wrap">';
    if ($separatorPos === SIMPLERWC_SEPARATOR_TOP) {
        $html .= simplerwc_button_separator_template();
    }

    $isSingleProduct = $positionId === SIMPLERWC_BTNPOS_PRODUCT_SHORTCODE || $positionId === SIMPLERWC_BTNPOS_PRODUCT_PAGE;
    $html .= simplerwc_button_template($id, $text, $isSingleProduct);

    if (simplerwc_should_render_cards_notice($positionId)) {
        $html .= simplerwc_accepted_cards_notice_template();
    }
    if ($separatorPos === SIMPLERWC_SEPARATOR_BOTTOM) {
        $html .= simplerwc_button_separator_template();
    }
    $html .= '</div></div>';
    return $html;
}

function simplerwc_marquee_button($id, $text, $positionId)
{
    $html = simplerwc_button_style($positionId);
    $html .= '<div class="simpler-container ' . $positionId . '">';
    $html .= '<div class="simpler-wrap simpler-marquee-wrap">';
    $html .= '<div class="simpler-marquee-top">Safer, Faster, Simpler</div>';
    $html .= simplerwc_button_template($id, $text, $positionId === SIMPLERWC_BTNPOS_PRODUCT_PAGE);
    if (simplerwc_should_render_cards_notice($positionId)) {
        $html .= simplerwc_accepted_cards_notice_template();
    }
    $html .= '<div class="simpler-marquee-bottom">OR</div>';
    $html .= '</div></div>';
    return $html;
}

function simplerwc_button_style($positionId)
{
    switch ($positionId) {
        case SIMPLERWC_BTNPOS_PRODUCT_PAGE:
            $style = get_option("simpler_product_button_custom_style");
            break;
        case SIMPLERWC_BTNPOS_MINICART:
            $style = get_option("simpler_mini_cart_button_custom_style");
            break;
        case SIMPLERWC_BTNPOS_PROCEED_TO_CHECKOUT:
            $style = get_option("simpler_cart_page_button_custom_style");
            break;
        case SIMPLERWC_BTNPOS_BEFORE_CHECKOUT:
            $style = get_option("simpler_checkout_page_button_custom_style");
            break;
        default:
            return false;
    }
    return '<style type="text/css">
        ' . esc_html($style) . '
    </style>';
}

function simplerwc_cart_contents_tag($id, $cart)
{
    $cartPayload = base64_encode(json_encode([
        'cart' => $cart,
        'currency' => get_woocommerce_currency(),
    ]));
    return '<input id="' . $id . '" type="hidden" value="' . esc_html($cartPayload) . '" />';
}

function simplerwc_prepare_product($product)
{
    return [
        simplerwc_get_product_attributes($product)
    ];
}

function simplerwc_prepare_cart($cart)
{
    return array_reduce($cart->get_cart_contents(), function ($acc, $el) {
        if (array_key_exists('data', $el) && $el['data'] instanceof WC_Product) {
            array_push($acc, simplerwc_get_product_attributes($el['data'], $el['quantity']));
        }
        return $acc;
    }, []);
}

function simplerwc_get_product_attributes(WC_Product $product, $qty = null)
{
    $attrs = [
        'product_id'         => strval($product->get_id()),
        'in_stock'           => $product->is_in_stock(),
        'backorders_allowed' => $product->backorders_allowed(),
        'stock_quantity'     => $product->get_stock_quantity()
    ];
    if (is_a($product, 'WC_Product_Simple')) {
        $attrs['product_type'] = 'simple';
    }
    if (is_a($product, 'WC_Product_Variable')) {
        $attrs['product_type'] = 'variable';
        $attrs['variations']   = array_map(function ($el) use ($qty) {
            return simplerwc_get_product_attributes(wc_get_product($el['variation_id']), $qty);
        }, $product->get_available_variations());
    }
    if (is_a($product, 'WC_Product_Variation')) {
        $attrs['product_type'] = 'variable';
        $attrs['variations']   = $product->get_variation_attributes();
    }

    if ($qty) {
        $attrs['quantity'] = $qty;
    }

    return $attrs;
}

function simplerwc_should_render_button()
{
    if (!simplerwc_check_simpler_configured()) {
        error_log('MISSING SIMPLER API KEY & SECRET');
        return false;
    }
    return !get_option('simpler_checkout_test_mode') || in_array('administrator', wp_get_current_user()->roles, true);
}

function simplerwc_should_render_cards_notice($positionId)
{
    switch ($positionId) {
        case SIMPLERWC_BTNPOS_PRODUCT_PAGE:
            return get_option('simplerwc_show_cards_notice_product_page');
        case SIMPLERWC_BTNPOS_MINICART:
            return get_option('simplerwc_show_cards_notice_minicart');
        case SIMPLERWC_BTNPOS_BEFORE_CHECKOUT:
        case SIMPLERWC_BTNPOS_PROCEED_TO_CHECKOUT:
            return get_option('simplerwc_show_cards_notice_checkout');
        default:
            return false;
    }
}
