<?php

namespace Simpler\Http\Payloads;

use Simpler\Models\Money;
use Simpler\Models\Quotation;

class QuotationResponse
{
    private $response = [];
    /**
     * @var Quotation[]
     */
    private $quotations;

    public function __construct(array $quotations)
    {
        $this->quotations = $quotations;
        $this->build_response();
    }

    private function build_response()
    {
        $this->response['quotes'] = [];
        foreach ($this->quotations as $quotation) {
            $this->response['quotes'][] = [
                'shipping'       => $quotation->get_shipping_rate() ? $this->get_shipping_rate_response(
                    $quotation->get_shipping_rate()
                ) : null,
                'discount_cents' => $quotation->get_discount_cents(),
                'total_cents'    => $quotation->get_total_cents(),
            ];
        }
    }

    /**
     * Get JSON representation of given shipping rate.
     *
     * @param  \WC_Shipping_Rate  $shippingRate
     *
     * @return array
     */
    private function get_shipping_rate_response(\WC_Shipping_Rate $shippingRate): array
    {
        return [
            'id'          => $shippingRate->get_id(),
            'method_id'   => $shippingRate->get_method_id(),
            'label'       => wc_clean($shippingRate->get_label()),
            'cost_cents'  => Money::to_cents($shippingRate->get_cost()),
            'instance_id' => $shippingRate->get_instance_id(),
        ];
    }

    public function to_array(): array
    {
        return $this->response;
    }
}
